/**
 * Views are a collection of filter group objects
 */
import versionNetworks from '../filterGroups/version-networks';
import versionMetadata from '../filterGroups/version-metadata';
import versionsMetadata from '../filterGroups/versions-metadata';
import versionDates from '../filterGroups/version-dates';
import versionSorting from '../filterGroups/version-sorting';

import search from '../filters/search';
import mapDictionary from  '../maps';
import config from '../config';
import _utils from '../_utils';
import TextItem from '../components/TextItem';
import filterDefaults from '../defaults';

export default {
    id:'versions',
    label:'Versions',
    enableRowChecks : false,
    writeQueryStringToURL: true,
    displayType : 'custom',//text[simple line item] or display[image & details line item]
    // Used to pass in the component when the displayType above is set to custom
    customDisplayTypeComponent : TextItem,//Component is passed < preferencedProps={preferencedProps} item={item} selectedView={selectedView}>
    enableGalleryLightbox : false,// Used by the 'display' displayType

    showTabsHeader :true,// filter view header
    showTabs :true,
    showListHeader : true,// List items header
    enableListSort:true,
    showResetFiltersButton : true,
    showListSettings : true,
    showSearch: true,

    infoDisplaySettings : {
        showIconStrip : true,
        showShareLink : true,
        showPaginationData : true,
        iconComponents:[]
    },

    /**
     * Filter groups to include in the view
     */
    filterGroups:[versionSorting,versionsMetadata,versionNetworks,versionDates],

    filterDefaults,

    /**
     * Need to specify the item property that is an item id
     */
    itemIdProp : 'entityUUID',

    /**
     * Entity types we are allowed to query/filter on for this view
     */
    listEntityTypes: ['olyplat-entity-version'],

    // @todo make render to a hook so we don't have to modify internals as needs arise
    renderTo:'store',//store is the default. IN the future we could render html, or to Meteor sessions, or anywhere else

    /**
     * Settings for how we make xhr requests
     */
    api:{
        url:config.versionsApiUrl,
        method:'POST',
        type:'rest',//rest,graphql

        /**
         * set to false to make non-tokened requests
         */
        token:localStorage.getItem(config.tokenLocalStorageKey)
    },

    paginationTake : 25,// Amount to take for our pages

    /**
     * This controls where you link to when you click a list row, and how it behaves
     */
    link:{
        row:item=> {
                return '#';
        },

        target:''//use '' or '_blank'
    },

    /**
     * Available item properties to the row. Also controls which props are visible by default or which are configurable
     * in the column settings
     *
     * `before` is a hook to transform the value (mostly for mapping) before rendering to the screen
     * `display` lets the datalist know that it should display that column on load. If it's false, it will not dipslay on load but will still be available to the column settings interface
     */
    props:[
        //@todo: if we define all props here that we want to make available in the column settings selector,
        // we can add a disable prop so it hides from the default render. This way we can control widths from outside the internal app
        {
            key:'state',
            label:'State',
            hasCopy:false,
            isDate:false,
            width:'5%',
            display: true,
            isSortable : false,                        
            before :val=>val
        },
        {
            key:'type',
            label:'Version Type',
            hasCopy:false,
            isDate:false,
            width:'10%',
            display: true,
            isSortable : true,                        
            before :val=>mapDictionary('versionsEntityType',val)
        },
        {
            key:'houseId',
            label:'House ID',
            hasCopy:true,
            isDate:false,
            width:'8%',
            display: true,
            isSortable : true,                        
            before :val=>val
        },
        {
            key:'titleCode',
            label:'Title Code',
            hasCopy:true,
            isDate:false,
            width:'12%',
            display: true,
            isSortable : true,                        
            before :val=>val
        },
        {
            key:'primaryNetworkLabel',
            label:'Network',
            hasCopy:false,
            isDate:false,
            width:'10%',
            display: true,
            isSortable : true,                        
            before :val=>val
        },
        {
            key:'entityType',
            label:'Entity Type',
            hasCopy:false,
            isDate:false,
            width:'8%',
            display: false,
            isSortable : true,                        
            before :val=>mapDictionary('catalogEntityType',val)
        },
        {
            key:'entityCreated',
            label:'Created',
            hasCopy:false,
            isDate: true,
            width:'12%',
            display: true,
            isSortable : true,                        
            before :val=>val
        },
        {
            key:'dateEdited',
            label:'Edited',
            hasCopy:false,
            isDate: true,
            width:'12%',
            display: false,
            isSortable : true,                        
            before :val=>val
        },
        {
            key: 'entityUUID',
            label: 'Id',
            hasCopy: false,
            isDate: false,
            width: '12%',
            display: false,
            isSortable : false,                        
            before: val=>val
        }
    ],

    /**
     * Addons are pseudo filter types that can be added to the view outside of a a filter group
     */
    addons : [
        search
    ],

    searchButton:{
        background: '#313541',
        text: '#fff'
    }
};